# Get content from the config.json
$config = Get-Content -Path .\config.json | ConvertFrom-Json

$xu_location = $config.xu_location
$xu_server = $config.xu_server
$xu_port = $config.xu_port
$end_of_upload_api = $config.end_of_upload_api

$extraction_start_date = $config.extraction_start_date
$extraction_end_date = $config.extraction_end_date
$language = $config.language
$exchange_rate_type = $config.exchange_rate_type

# Create a new log file for every run
$logFile = ".\extraction.log"

if (Test-Path -Path .\$logFile -PathType Leaf) {
    Remove-Item .\$logFile
}

# Function for appending text to the log file
function WriteLog
{
Param ([string]$logText)
$timestamp = (Get-Date).toString("dd/MM/yyyy HH:mm:ss")
Add-content $logFile -value "$timestamp $logText"
Write-Output $logText
}

# Function for checking the error code of extractions
function CheckExtractionError
{
Param ([string]$extraction)
if ($LASTEXITCODE -ne 0) {
    WriteLog("$extraction failed with return code: $LASTEXITCODE")
    Set-Variable -Name "extraction_failed" -Value 1 -Scope global
}
}

# If at least one extraction fails, do not upload to blob storage.
Set-Variable -Name "extraction_failed" -Value 0 -Scope global

# Extraction commands
# Transactional tables
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_LIKP" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('WM_LIKP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_LIPS" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('WM_LIPS')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_LTAK" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('WM_LTAK')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_LTAP" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('WM_LTAP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_LTBK" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('WM_LTBK')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_LTBP" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('WM_LTBP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_MKPF" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('WM_MKPF')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_MSEG" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('WM_MSEG')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_CDHDR" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('WM_CDHDR')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_CDPOS" -o "extraction_start_date=$extraction_start_date" -o "extraction_end_date=$extraction_end_date"
CheckExtractionError('WM_CDPOS')

# Master data tables 
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_ADCP"
CheckExtractionError('WM_ADCP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_ADRP"
CheckExtractionError('WM_ADRP')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_MAKT" -o "language=$language"
CheckExtractionError('WM_MAKT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_MBEW"
CheckExtractionError('WM_MBEW')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T001"
CheckExtractionError('WM_T001')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T001K"
CheckExtractionError('WM_T001K')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T001W"
CheckExtractionError('WM_T001W')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T003T" -o "language=$language"
CheckExtractionError('WM_T003T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T006A" -o "language=$language"
CheckExtractionError('WM_T006A')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T156"
CheckExtractionError('WM_T156')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T156T" -o "language=$language"
CheckExtractionError('WM_T156T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T158W" -o "language=$language"
CheckExtractionError('WM_T158W')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T300"
CheckExtractionError('WM_T300')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T300T" -o "language=$language"
CheckExtractionError('WM_T300T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T301T" -o "language=$language"
CheckExtractionError('WM_T301T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T302T" -o "language=$language"
CheckExtractionError('WM_T302T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T308T" -o "language=$language"
CheckExtractionError('WM_T308T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T333B" -o "language=$language"
CheckExtractionError('WM_T333B')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T333T" -o "language=$language"
CheckExtractionError('WM_T333T')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_T646S" -o "language=$language"
CheckExtractionError('WM_T646S')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_TCURX"
CheckExtractionError('WM_TCURX')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_TVAPT" -o "language=$language"
CheckExtractionError('WM_TVAPT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_TVLKT" -o "language=$language"
CheckExtractionError('WM_TVLKT')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_USR02"
CheckExtractionError('WM_USR02')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_USR21"
CheckExtractionError('WM_USR21')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_TTZCU"
CheckExtractionError('WM_TTZCU')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_TTZD"
CheckExtractionError('WM_TTZD')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_TTZDF"
CheckExtractionError('WM_TTZDF')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_TTZDV"
CheckExtractionError('WM_TTZDV')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_TTZR"
CheckExtractionError('WM_TTZR')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_TTZZ"
CheckExtractionError('WM_TTZZ')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_TCURF" -o "exchange_rate_type=$exchange_rate_type"
CheckExtractionError('WM_TCURF')
&"$xu_location\xu.exe" -s "$xu_server" -p "$xu_port" -n "WM_TCURR" -o "exchange_rate_type=$exchange_rate_type"
CheckExtractionError('WM_TCURR')

# Only ingest data if extraction is successful.
if ($extraction_failed) {
    Exit
} else {
    Invoke-WebRequest -Method POST -Uri $end_of_upload_api
}
